/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ParamValuesSelectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");
dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ParamValuesSelectionDialogMessages");

(function() {

var DELAY = 500;
var KEY_ESCAPE = 27;
var KEY_RETURN = 13;
var KEY_ENTER = 14;
	
/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.ParamValuesSelectionDialog", [dijit._Widget, dijit._Templated], {
	
	/*
	 * ParamValuesSelectionDialog is a dialog used to filter and select one or more values from a list. 
	 * The dialog starts with an empty list and shows matched values as the user enters a filter string. 
	 * "*" and "?" are supported as wild card in the filter. Users can then select the desired value/values
	 * from the list.
	 * 
	 * The list of values available are taken from the constructor argument of this widget.  Thus this is
	 * a pure client side dialog, no server interactions. Can be configured to allow multi-selection or single-selection. 
	 * The following arguments are supported:
	 * 
	 * values (required, array): The full list of values to be filtered and displayed. Each value is an object with minimally 
	 * 		two attributes: "itemId" and "label". e.g. 
	 *  	{itemId: b3cc35208d96fa7d0864531ce5a27c6e, 
	 *  	 label: "April"}
	 *  
	 * allowMultiple (boolean): whether to allow multiple selections, default true.
	 * 
	 * useOkCancel (boolean): If true, "OK" and "Cancel" buttons will be used in this dialog. 
	 * 		If false, "Add and Close", "Add" and "Close" buttons will be used. 
	 * 		Default is false.
	 * 
	 * dialogTitle (optional): If not specified, the default is "Select Value" or "Select Values" depending on argument allowMultiple.
	 *  
	 * width (string): width of the dialog, default is "450px".
	 * 
	 * isClosable (boolean): default true.
	 * 
	 * removeValuesOnAdd (boolean): default false.
	 * 
	 * disableValuesOnAdd (boolean): default true.
	 * 
	 * disableValues (array of string): used to keep the "itemId" of values that need to be shown as disabled.
	 */
	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ParamValuesSelectionDialog.html"),
	
	lastSearchText: null,
	
	constructor: function (args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ParamValuesSelectionDialogMessages");
		if (args == null) {
			args = {};
		}
		this.width = args.width || "450px";
		var properties = {
			"values": [],
			"dialogTitle" : args.allowMultiple == null || args.allowMultiple ? this.messages.selectValues : this.messages.selectValue,
			"width" : "450px",
			"isClosable": true,
			"removeValuesOnAdd": false,
			"disableValuesOnAdd": true,
			"filterOutValues" : [ ],
			"disableValues" : [ ],
			"allowMultiple" : true,
			"useOkCancel" : false
		};
		for (var prop in properties) {
			if (args[prop] != null) {
				this[prop] = args[prop];
			}
			else {
				this[prop] = properties[prop];
			}
		}
		this.selectedUsers = [];
	},
	
	postCreate: function () {
		this.searchCount = 0;
		this.userSelector.size = 15;
		this.keyStrokeFilter = dojo.connect(document, "onkeypress", this, function (e) {
			if (e.keyCode === 8 && e.target !== this.searchText) {
				dojo.stopEvent(e);
			}
		});
		if (this.useOkCancel) {
			this.addCloseButton.appendChild(document.createTextNode(this.messages.ok));
			this.closeButton.appendChild(document.createTextNode(this.messages.cancel));
			this.addCloseButton.style.display = "";
			this.closeButton.style.display = "";
		}
		else {
			this.addCloseButton.appendChild(document.createTextNode(this.messages.addAndClose));
			this.addButton.appendChild(document.createTextNode(this.messages.add));
			this.closeButton.appendChild(document.createTextNode(this.messages.close));
			this.addCloseButton.style.display = "";
			this.addButton.style.display = "";
			this.closeButton.style.display = "";
		}
		this.userSelector.multiple = this.allowMultiple;
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: this.width,
			isClosable: this.isClosable
		});
		this.connect(this.searchText,"onkeypress","handleSearchChange");
		this.connect(this.searchText,"onchange","handleSearchChange");
		this.connect(this.showAllLink,"onclick", function (e) {
			dojo.stopEvent(e);
			this.searchText.value = "*";
			this._doSearch();
		});
		this.connect(this.closeButton,"onclick", function (e) {
			dojo.stopEvent(e);
			this.dialog.close();
		});
		this.connect(this.addCloseButton,"onclick","_onAddClose");
		this.connect(this.addButton,"onclick","_onAdd");
		this.connect(this.userSelector,"ondblclick","_handleDoubleClick");
		this.connect(this.userSelector,"onkeydown","_handleKeyDown");
		// compensate for the IE bug that prevents us from disabling options
		if (dojo.isIE) {
			this.connect(this.userSelector,"onchange", function(e) {
				var selectedIndices = [];
				var disabledSelected = false;
				if (this.allowMultiple) {
					for (var i=0; i < this.userSelector.options.length; i++) {
						if (this.userSelector.options[i].selected) {
							if (this.userSelector.options[i].disabled) {
								disabledSelected = true;
								this.userSelector.options[i].selected = false;
							}
							else {
								selectedIndices.push(i);
							}
						}
					}
					if (disabledSelected) {
						if (selectedIndices.length > 0) {
							this.userSelector.savedSelected = selectedIndices;
						}
						else {
							for (var i=0; i < this.userSelector.savedSelected.length; i++) {
								this.userSelector.options[this.userSelector.savedSelected[i]].selected = true;
							}
						}
					}
					else {
						this.userSelector.savedSelected = selectedIndices;
					}
				}
				else {
					if (this.userSelector.selectedIndex >= 0) {
						if (this.userSelector.options[this.userSelector.selectedIndex].disabled) {
							if (this.userSelector.savedSelectedIndex >= 0) {
								this.userSelector.selectedIndex = this.userSelector.savedSelectedIndex;
							}
							else {
								this.userSelector.selectedIndex = -1;
							}
						}
						else {
							this.userSelector.savedSelectedIndex = this.userSelector.selectedIndex;
						}
					}
				}
			});
		}
		this.connect(this.userSelector,"onchange", function () {
			var selected = false;
			this.selectedUsers = [];
			for (var i=0; i < this.userSelector.options.length; i++) {
				if (this.userSelector.options[i].selected) {
					selected = true;
					this.selectedUsers.push({
						itemId: this.userSelector.options[i].value,
						label: this.userSelector.options[i].label
					});
				}
			}
			this.addCloseButton.disabled = !selected;
			this.addButton.disabled = !selected;
		});
		this.connect(this.dialog,"onClose","onClose");
		this.searchText.focus();
	},
	
	_handleDoubleClick: function (e) {
		if (this.selectedUsers.length > 0) {
			this._onAddClose(e);
		}
	},
	
	_handleKeyDown: function (e) {
		if ((e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) && this.selectedUsers.length > 0) {
			this._onAddClose(e);
		}
	},
	
	handleSearchChange: function (e) {
		if(e.keyCode == KEY_ESCAPE) {
			return;
		}
		if (e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) {
			if (this.selectedUsers.length == 0 && this.userSelector.options.length > 0) {
				var index = 0;
				while (index < this.userSelector.options.length && this.userSelector.options[index].disabled) {
					index++;
				}
				if (index != this.userSelector.options.length) {
					this.userSelector.options[index].selected = true;
					this.selectedUsers.push({
						itemId: this.userSelector.options[index].value,
						label: this.userSelector.options[index].label
					});
				}
			}
			if (this.selectedUsers.length > 0) {
				this._onAddClose(e);
			}
		}
		else {
			this._doCancellableSearch();
		}
	},
	
	_onAddClose: function (e) {
		dojo.stopEvent(e);
		this.onAdd(this.selectedUsers);
		this.dialog.close();
	},
	
	_onAdd: function (e) {
		dojo.stopEvent(e);
		this.onAdd(this.selectedUsers);
	},
	
	// an equivalent to onAdd that can be connected to.
	onOK: function (selectedUsers) {},
	
	onAdd: function (selectedUsers) {
		this.selectedUsers = [];
		this.addCloseButton.disabled = true;
		this.addButton.disabled = true;
		for (var i=0; i < this.userSelector.options.length; i++) {
			if (this.userSelector.options[i].selected) {
				this.userSelector.options[i].selected = false;
				if (this.removeValuesOnAdd) {
					this.filterOutValues.push({'itemId': this.userSelector.options[i].value});
					if (dojo.isIE) {
						// IE doesn't handle removeChild and insertBefore well for select lists
						// Do things the old-fashioned way
						var option = this.userSelector.options[i];
						var opt = option;
						for (var j=i; j < this.userSelector.options.length - 1; j++) {
							var curOpt = this.userSelector.options[j+1];
							var newOpt = new Option(curOpt.text, curOpt.value);
							//set the "label" attribute separately - seems that IE does not set the label in the constructor, tho other browsers do
							newOpt.label = curOpt.text;
							this.userSelector.options[j] = newOpt;
						}
						this.userSelector.options.length = this.userSelector.options.length - 1;
					}
					else {
						this.userSelector.removeChild(this.userSelector.options[i]);
					}
				}
				else if (this.disableValuesOnAdd) {
					this.disableValues.push({'itemId': this.userSelector.options[i].value});
					if (dojo.isIE) {
						this.userSelector.options[i].style.color = "gray";
						if (this.userSelector.savedSelected != null && this.userSelector.savedSelected.length > 0) {
							this.userSelector.savedSelected = [];
						}
					}
					this.userSelector.options[i].disabled = "disabled";
				}
			}
		}
		this.onOK(selectedUsers);
	},
	
	onClose: function () {
		if (this.keyStrokeFilter !== null) {
			dojo.disconnect(this.keyStrokeFilter);
		}
	},
	
	_doCancellableSearch: function() {
		if (this._currentId) {
			clearTimeout(this._currentId);
		}
		this._currentId= setTimeout(dojo.hitch(this, function () {this._doSearch();}), DELAY);
	},
	
	_doSearch: function () {
		// if there's no input, or the input hasn't changed since the last search, do nothing
		var value = this.searchText.value;
		if (dojo.trim(value) === "" || value === this.lastSearchText) {
			this._clearStatusMessage();
			return;
		}

		this.lastSearchText = value;
		value = value.replace("*", ".*");
		value = value.replace("?", ".");
		value = new RegExp(value);
		
		if (this.latestSearchTerm != null && this.latestSearchTerm === value) {
			// don't re-run the same search twice
			return;
		}
		this.latestSearchTerm = value;		
		this._filterValues(value);
	},
	
	_filterValues: function (filterText) {
		var disabledItemIdHash = {};
		for (var i=0; i < this.disableValues.length; i++) {
			if (this.disableValues[i].itemId != null) {
				disabledItemIdHash[this.disableValues[i].itemId] = true;
			}
		}
		// compensate for the IE bug that prevents us from disabling options
		if (dojo.isIE) {
			this.userSelector.savedSelected = [];
		}
		if (dojo.isIE < 7) {
			this.userSelector.options.length = 0;
		}
		else {
			dojox.xml.parser.removeChildren(this.userSelector);
		}
		if (this.values != null) {
			var enableAdd = false;
			for (var i=0; i < this.values.length; i++) {
				var disp = this.getDispValue(this.values[i]);
				if(disp.search(filterText) == -1)
					continue;
				var option = new Option(disp, this.values[i].itemId);
				//set the "label" attribute separately - seems that IE does not set the label in the constructor, tho other browsers do
				option.label = this.values[i].label;
				if (disabledItemIdHash[this.values[i].itemId]) {
					option.disabled = "disabled";
					// compensate for the IE bug that prevents us from disabling options
					if (dojo.isIE) {
						option.style.color = "gray";
					}
				}
				else
					enableAdd = true;
					
				if (dojo.isIE) {
					this.userSelector.options[this.userSelector.options.length] = option;
				}
				else {
					this.userSelector.appendChild(option);
				}
			}
			if(enableAdd) {
				this.addCloseButton.disabled = true;
				this.addButton.disabled = true;
			}
		}
		else {
			this.addCloseButton.disabled = true;
			this.addButton.disabled = true;
		}
		this._clearStatusMessage();
	},
	
	getDispValue: function (elem) {
		return elem.label;
	},
	
	_clearStatusMessage: function() {
		this.searchStatus.style.visibility = "hidden";
	}
	
//	_setStatusMessage: function(msg, isError) {
//		this.searchStatus.style.visibility = "visible";
//		dojox.data.dom.replaceChildren(this.searchStatus, document.createTextNode(msg));
//		this.searchStatus.className= isError ? "ErrorStatus" : "NormalStatus";
//	}
});

})();