/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogDatesCalculator"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

	var PROP_START_DATE = "startDate"; //$NON-NLS-1$
	var PROP_END_DATE = "endDate"; //$NON-NLS-1$
	var PROP_ITERATION_STRUCTURE_ID = "iterationStructureId"; //$NON-NLS-1$
	var PROP_DURATION = "duration"; //$NON-NLS-1$

	/**
	 * Utility to calculate the start-end dates for preview items in the
	 * CreateIterationsDialog.
	 * 
	 * @since 4.0.5
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogDatesCalculator", null, { //$NON-NLS-1$

		/**
		 * Required arguments:
		 * 
		 * @param previewModel :
		 *            The mode containing the store which contains all the
		 *            preview items, that need to be updated.
		 * @param iterationStructureModel :
		 *            The model (containing the store) for the all iteration
		 *            structure table elements.
		 * @param commonStartDate :
		 *            The start date from which all calculations begin.
		 * @param scheduledDays :
		 *            An array of 7 boolean elements, each of which represents
		 *            the checked box for selected days [Su, Mo, Tu, We, Th, Fr,
		 *            Sa]
		 * 
		 */
		constructor: function(args) {
			this._requestedStartDate = args.commonStartDate;
			this._scheduledDays = args.scheduledDays;
			this._model = args.previewModel;
			this._store = this._model.store;
			this._iterationStructureModel = args.iterationStructureModel;

			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

		},

		/**
		 * Sets new common start date, which is to be used in calculations.
		 * 
		 * @param newStartDate
		 */
		setCommonStartDate: function(newCommonStartDate) {
			this._requestedStartDate = newCommonStartDate;
		},

		/**
		 * Sets the selection of scheduled days, which will be used in
		 * calculations.
		 * 
		 * @param scheduledDays :
		 *            An array of 7 boolean elements, each of which represents
		 *            the checked box for selected days [Su, Mo, Tu, We, Th, Fr,
		 *            Sa]
		 */
		setScheduledDays: function(newScheduledDays) {
			this._scheduledDays = newScheduledDays;
		},

		/**
		 * Updates the dates for the create iterations preview tree (which was
		 * provided in the constructor. <br>
		 * We should always start with the root item, because it's end date may
		 * be updated by changing the duration of child items.
		 * 
		 * @returns : A map ( preview item identity -> warning message ) for all
		 *          items for which the date calculation ended with a warning
		 *          message.
		 */
		updateDates: function() {
			var warningReports = {};
			this._updateChildrenDates(warningReports, this._model.root, this._adjustStartDate(this._requestedStartDate), false);

			return warningReports;
		},

		/**
		 * Updates the dates for children of the the passed items and all their
		 * children. <br>
		 * For every previewItem and it's children this function will set the
		 * properties "startDate" and "endDate" to their newly calculated
		 * values. If the dates cannot be calculated their values will be set to
		 * null.
		 * 
		 * @param warningReport :
		 *            A map ( preview item identity -> warning message ) with
		 *            all items for which the date calculation ended with a
		 *            warning message.
		 * @param previewItem :
		 *            The preview item from which the dates must be
		 *            recalculated.
		 * @param startDate :
		 *            The pre-calculated start date, which should be used for
		 *            the item.
		 * @param parentsDurationIsZero :
		 *            true when the duration of one of the parents is 0, false
		 *            otherwise.
		 */
		_updateChildrenDates: function(warningReport, item, startDate, parentsDurationIsZero) {
			var theEndDate = null;
			var durationIsZero = parentsDurationIsZero;
			var childStartDate = startDate;

			if (this._isNewItem(item)) {
				var duration = this._getDuration(item);
				childStartDate = (duration === 0) ? null : startDate;
				durationIsZero = durationIsZero || duration === 0;
				this._setDate(item, PROP_START_DATE, childStartDate);
				theEndDate = this._calculateEndDate(childStartDate, duration);
			} else {
				theEndDate = this._store.getValue(item, PROP_END_DATE);
			}

			// Recursively process the children
			var children = this._store.getValues(item, "items"); //$NON-NLS-1$
			var calculatedEndDate = null;
			var proposedEndDate = null;
			for ( var i = 0; i < children.length; i++) {
				var child = children[i];
				if (this._isNewItem(child)) {
					var childDuration = this._getDuration(child);

					calculatedEndDate = this._updateChildrenDates(warningReport, child, childStartDate, durationIsZero);

					if (calculatedEndDate && theEndDate && theEndDate < calculatedEndDate) {
						// Child end date went outside of this parent's end date. 
						// Report a warning.
						var childId = this._model.getIdentity(child);
						warningReport[childId.toString()] = this.messages.CreateIterationsDialogCreateIterationsDialogDatesCalculator_1;
					}

					if (!calculatedEndDate && childDuration) {
						if (durationIsZero) {
							// Warn that the end date cannot be calcucated because the duration of the parent is set to 0.
							var childId = this._model.getIdentity(child);
							warningReport[childId.toString()] = this.messages.CreateIterationsDialogCreateIterationsDialogDatesCalculator_3;
						} else if (this._requestedStartDate) {
							// Warn that the end date cannot be calculated because the duration of previous sibling was not set.
							var childId = this._model.getIdentity(child);
							warningReport[childId.toString()] = this.messages.CreateIterationsDialogCreateIterationsDialogDatesCalculator_2;
						}
					}

					if (calculatedEndDate != null) {
						childStartDate = this._adjustStartDate(new Date(calculatedEndDate.getFullYear(), calculatedEndDate.getMonth(), calculatedEndDate.getDate() + 1));
						proposedEndDate = calculatedEndDate;
					} else {
						if (childDuration != 0) {
							childStartDate = calculatedEndDate;
						}
					}
				}
			}

			if (this._isNewItem(item)) {
				if (theEndDate == null) {
					theEndDate = proposedEndDate;
				}
				this._store.setValue(item, PROP_END_DATE, theEndDate);
			}
			return theEndDate;
		},

		/**
		 * Finds the first day starting from date matching this._scheduledDays
		 * 
		 * @param date:
		 *            The date to start the search from
		 * @returns the new adjusted date.
		 */
		_adjustStartDate: function(date) {
			if (date) {
				var i = date.getDay();
				for ( var days = 0; days < 7; days++) {
					if (this._scheduledDays[i]) {
						if (days === 0) {
							//No shift needed
							return date;
						} else {
							return new Date(date.getFullYear(), date.getMonth(), date.getDate() + days);
						}
					}
					i = ++i % this._scheduledDays.length;
				}
			}
			return null;
		},

		_calculateEndDate: function(startDate, duration) {
			var endDate = null;
			if (startDate && duration) {
				endDate = new Date(startDate.getFullYear(), startDate.getMonth(), startDate.getDate() + (7 * duration - 1));

				var i = endDate.getDay();
				for ( var days = 0; days < 7; days++, i--) {
					if (i < 0) {
						i = 6;
					}
					if (this._scheduledDays[i]) {
						if (days === 0) {
							//No shift needed
							return endDate;
						} else {
							return new Date(endDate.getFullYear(), endDate.getMonth(), endDate.getDate() - days);
						}
					}
				}
			}
			return endDate;
		},

		_getDuration: function(item) {
			var duration = undefined;
			if (item) {
				var iterationStructureId = this._store.getValue(item, PROP_ITERATION_STRUCTURE_ID);
				var structureItem = this._iterationStructureModel.store.fetchItemByIdentity(iterationStructureId);
				duration = this._iterationStructureModel.store.getValue(structureItem, PROP_DURATION);
			}
			return duration;
		},

		_isNewItem: function(item) {
			var existing = this._store.getValue(item, "existing");//$NON-NLS-1$
			return (existing != true);
		},

		_setDate: function(item, prop, date) {
			this._store.setValue(item, PROP_START_DATE, date);
		}
	});
})();
