/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy"); //$NON-NLS-1$
	
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var PermittedType = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType;
	
	var PROP_PARENT_ID = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_PARENT_ID;
	var PROP_PERMISSION = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_PERMISSION;
	var PROP_DENIAL_EXPLANATION_CONFIGURED = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_DENIAL_EXPLANATION_CONFIGURED;
	var PROP_HAS_ONLY_PROJECT_OPERATIONS = "hasOnlyProjectOperations"; //$NON-NLS-1$  
	
	var SAVE_FIELD_GRANTED = "granted"; //$NON-NLS-1$
	var SAVE_FIELD_REVOKED = "revoked"; //$NON-NLS-1$
	var SAVE_FIELD_OVERRIDE_CLEARED = "overrideCleared"; //$NON-NLS-1$
	var SAVE_FIELD_FINAL = "final"; //$NON-NLS-1$
	var SAVE_FIELD_EXPLANATION = "explanation"; //$NON-NLS-1$
	var SAVE_FIELD_ACTION_PATHS = "actionPaths"; //$NON-NLS-1$
	
	/**
	 * An object of this type is to be common among the various 'editors' for
	 * permissions. It encapsulates the following functionality:
	 * 
	 * - Loading configurable actions (aka permission extensions) from the
	 *   server
	 * - Loading permitted/not permitted actions for a given role or set of
	 *   roles, as an asynchronous operation
	 * - Loading and tracking for which permissions scopes permission are
	 *   configured for each role, to answer which roles are configured in a
	 *   given scope and which scopes are configured for a given role
	 * - Answering whether or not a given role is permitted to execute a
	 *   given action, as a synchronous operation
	 * - Saving updates to permitted actions to the server 
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy", [], { //$NON-NLS-1$
		
		/**
		 * Constructs the PermissionsManager instance
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 * 	  editorContext			One of ProcessConstants.EditorContext. Used
		 * 							in inherited permissions calculation
		 * 	  getProcessAreaTitle	Function that returns name of project/team
		 * 							whose permissions are tracked. Used in
		 * 							inherited permissions calculation 
		 *    isChildArea			boolean
		 *    processAreaItemId		string
		 *    owningApplicationKey	string
		 *    rolesStore			com.ibm.team.process.web.ui.internal.admin.projects.RolesStore for the editor
		 * 	  processAreaStore		store with timeline/iteration/iteration type items, (aka permissions scopes) for the editor 
		 *    startLoadingFunction	function to notify editor to enter the loading state with a given message
		 *    stopLoadingFunction	function to notify editor to exit the loading state
		 *  }
		 */
		constructor: function(args) {
			this._editorContext = args.editorContext;
			this._getProcessAreaTitle = args.getProcessAreaTitle;
			this._isChildArea = args.isChildArea;
			this._processAreaItemId = args.processAreaItemId;
			this._owningApplicationKey = args.owningApplicationKey;
			this._rolesStore = args.rolesStore;
			this._processAreaStore = args.processAreaStore;
			this._startLoadingFunction = args.startLoadingFunction;
			this._stopLoadingFunction = args.stopLoadingFunction;
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._storeConnects = [];
			
			this.clearState();
		},
		
		/**
		 * Loads the categories, operations, and actions for which
		 * permissions can be configured from the server, if necessary.
		 * Whether data is retrieved from the server or returned from memory,
		 * the provided function successCallback is invoked with a
		 * dojo.data.ItemFileReadStore which contains the hierarchy of permission
		 * extensions.
		 * 
		 * This function must be called and its data loaded successfully for
		 * the function loadPermissions() to be used.
		 * 
		 * @param	successCallback	function (
		 * @param	errorCallback	function
		 */
		loadPermissionExtensions: function(successCallback, errorCallback) {
			if (this._permissionExtensionsStore === undefined) {
				this._loadingPermissionExtensionsSuccessCallback = successCallback;
				this._loadingPermissionExtensionsErrorCallback = errorCallback;
				
				this._startLoadingFunction(this.messages.PermissionsWorkingCopy_1);
				
				this._loadingPermissionExtensions += 1;				
				if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
					this._loadingPermissionExtensions += 1;
					this._getPermissionExtensionsFromServer(ProcessConstants.EditorContext.PROJECT, this._processAreaItemId, this._owningApplicationKey, "_populatePermissionExtensions", "_handlePermissionExtensionsFetchError"); //$NON-NLS-2$ //$NON-NLS-1$
				}
				this._getPermissionExtensionsFromServer(ProcessConstants.EditorContext.TEAM, this._processAreaItemId, this._owningApplicationKey, "_populatePermissionExtensions", "_handlePermissionExtensionsFetchError"); //$NON-NLS-2$ //$NON-NLS-1$
				
			} else {
				setTimeout(dojo.hitch(this, function() {
					successCallback(this._permissionExtensionsStore);
				}));
			}
		},
			
		_getPermissionExtensionsFromServer: function(context, processAreaItemId, owningApplicationKey, successCallback, errorCallback) {
			var srh = new ServiceResponseHandler(this, successCallback, errorCallback); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getPermissionExtensions({
				context: context,
				processAreaItemId: processAreaItemId,
				owningApplicationKey: owningApplicationKey
			}, srh);
		},
		
		_populatePermissionExtensions: function(data) {
			//If this._loadingPermissionExtensions is 0, an error has
			//occurred with a different call to service /permissionExtensions.
			//Disregard this response that wasn't an error.
			if (this._loadingPermissionExtensions !== 0) {
				
				var addedCategory = false;
				
				var processChunk = dojo.hitch(this, function() {
					for (var i = 0; i < 5 && data.length !== 0; i++) {
						var category = data.shift();
						category[PROP_HAS_ONLY_PROJECT_OPERATIONS] = true;
						
						if (category.items && category.items.length > 0) {
							for (var j = 0; j < category.items.length; j++) {
								var operation = category.items[j];
								operation[ProcessConstants.PermissionConfigurationDTO.FINAL_OP] = false;
								operation[ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION] = null;
								if (!this._processPermissionExtensionChild(operation, category, operation, category)) {
									// duplicate extension, don't put it in the store
									category.items.splice(j--, 1);
								}
							}
						}
						
						var existingCategory = this._permissionExtensionCategoriesByName[category.name];
						if (existingCategory === undefined) {
							this._permissionExtensionCategoriesByName[category.name] = category;
							this._permissionExtensionsData.items.push(category);
							addedCategory = true;							
							if (category.items && category.items.length > 0) {
								category.items.sort(function(a, b){
									return ProcessClient.stringCompare(a.name, b.name);
								});
							}
						} else {
							if (category.items && category.items.length > 0) {
								if (!existingCategory.items) {
									existingCategory.items = category.items;
								} else {
									existingCategory.items = existingCategory.items.concat(category.items);
								}
								if (category[PROP_HAS_ONLY_PROJECT_OPERATIONS] === false) {
									existingCategory[PROP_HAS_ONLY_PROJECT_OPERATIONS] = false;
								}								
								dojo.forEach(category.items, function(operationItem) {
									operationItem[PROP_PARENT_ID] = existingCategory.id;
								});
								
								existingCategory.items.sort(function(a, b){
									return ProcessClient.stringCompare(a.name, b.name);
								});
							}
						}
					}
					
					if (data.length === 0) {
						if (addedCategory) {
							this._permissionExtensionsData.items.sort(function(a, b){
								return ProcessClient.stringCompare(a.name, b.name);
							});
						}
						
						this._loadingPermissionExtensions -= 1;
						if (this._loadingPermissionExtensions === 0) {
		                	this._permissionExtensionsStore = new dojo.data.ItemFileReadStore({
								data: this._permissionExtensionsData
							});				
							delete this._permissionExtensionsData;
							
							this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_1);					
							this._loadingPermissionExtensionsSuccessCallback(this._permissionExtensionsStore);
						}
					} else {
						setTimeout(processChunk);
					}
				});
				
				setTimeout(processChunk);
			}
		},
		
		_processPermissionExtensionChild: function(operation, category, parentOperation, parent) {
			operation.id = category.id + ProcessConstants.PERMISSION_PATH_SEPARATOR + operation.id;
			operation[PROP_PARENT_ID] = parent.id;
			
			if (this._permissionExtensionsIdMap[operation.id]) {
				// duplicate permission extension, log to the console and return
				// false to indicate that this extension should be ignored
				if (console && console.error) {
					console.error(new Error("Duplicate permission extension found: " + operation.id)); //$NON-NLS-1$
				}
				return false;
			}
			this._permissionExtensionsIdMap[operation.id] = true;
			if (operation.items && operation.items.length > 0) {
				operation.items.sort(function(a, b){
					return ProcessClient.stringCompare(a.name, b.name);
				});
				for (var i = 0; i < operation.items.length; i++) {
					if (!this._processPermissionExtensionChild(operation.items[i], category, parentOperation, operation)) {
						// duplicate extension, don't put it in the store
						operation.items.splice(i--, 1);
					}
				}
			}
			if (parent === category && ProcessConstants.ExtensionContext.TEAM === operation[ProcessConstants.ExtensionDTO.CONTEXT]) {
				category[PROP_HAS_ONLY_PROJECT_OPERATIONS] = false;
			}
			return true;  
		},
		
		_handlePermissionExtensionsFetchError: function(error) {
			//If this._loadingPermissionExtensions is 0, an error has
			//occurred with a different call to service /permissionExtensions.
			//Disregard this subsequent response that was an error.
			if (this._loadingPermissionExtensions !== 0) {
				this._loadingPermissionExtensions = 0;
				this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_1);
				this._loadingPermissionExtensionsErrorCallback(error);
			}
		},
		
		/**
		 * Loads the set of permissions for a particular role and time period
		 * from the server, if necessary. If data is successfully retrieved from
		 * the or it has already been loaded, the provided function successCallback
		 * is invoked with no arguments.
		 * 
		 * @param	roleId		The identifier for a role
		 * @param	contextItem	Item in the process area store for the time
		 * 						period to load permissions for
		 * @param	successCallback	function                   
		 * @param	errorCallback	function( error ) 
		 */
		loadPermissions: function(roleId, contextItem, successCallback, errorCallback) {
			if (this._permissionExtensionsStore === undefined) {
				throw new Error("Before loading permissions, permission extension data must be loaded via function loadPermissionExtensions()"); //$NON-NLS-1$
			}
			
			if (this._loadedPermissions[this._getStoreId(roleId, contextItem)]) {
				setTimeout(function() {
					successCallback();
				});
			} else {
				if (this._pendingPermissions) {
					throw new Error(dojo.string.substitute("Already loading permissions for ${0}", [JSON.stringify(this._pendingPermissions)])); //$NON-NLS-1$
				}
				
				this._loadPermissionsSuccessCallback = successCallback;
				this._loadPermissionsErrorCallback = errorCallback;
				
				this._pendingPermissions = {
					roleId: roleId,
					contextItem: contextItem
				};
								
				var roleItem = this._rolesStore.fetchRoleById(roleId);
				var roleLabel = this._rolesStore.getRoleLabel(roleItem);
				this._loadingPermissionsMessage = dojo.string.substitute(this.messages.PermissionsWorkingCopy_2, [roleLabel]);
				this._startLoadingFunction(this._loadingPermissionsMessage);
				
				var fetchPermissions = dojo.hitch(this, function(context, contextId) {
					var srh = new ServiceResponseHandler(this, "_handlePermissions", "_handlePermissionsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
					this._getPermissionsFromServer(context, contextId, roleId, "_handlePermissions", "_handlePermissionsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				});
				
				this._loadingPermissions += 1;
				if (contextItem === this._processAreaStore.alwaysRootItem) {
					if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
						this._loadingPermissions += 1;
						fetchPermissions(ProcessConstants.PermissionContext.PROJECT, ""); //$NON-NLS-1$
					}
					fetchPermissions(ProcessConstants.PermissionContext.TEAM, ""); //$NON-NLS-1$
				} else {
					fetchPermissions(this._processAreaStore.getValue(contextItem, "enabledContext"), this._processAreaStore.getSavedContextId(contextItem)); //$NON-NLS-1$
				}
			}
		},
		
		_getPermissionsFromServer: function(context, contextId, roleId, successCallback, errorCallback ) {
			var srh = new ServiceResponseHandler(this, successCallback, errorCallback); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getPermissions({
				context: context,
				contextId: contextId,
				owningApplicationKey: this._owningApplicationKey,
				processAreaContext: this._editorContext,
				processAreaItemId: this._processAreaItemId,
				roleId: roleId
			}, srh);	
		},
		
		_handlePermissions: function(data) {
			if (this._loadingPermissions !== 0) {
				this._loadingPermissions -= 1;
				this._populatePermissions(this._pendingPermissions.roleId, data, this._pendingPermissions, this._loadingPermissions);
				if (this._loadingPermissions === 0) {
					this._stopLoadingFunction(this._loadingPermissionsMessage);
					delete this._pendingPermissions;
					this._loadPermissionsSuccessCallback();
				}
			}
		},
		
		_handlePermissionsFetchError: function(error) {
			if (this._loadingPermissions !== 0) {
				this._stopLoadingFunction(this._loadingPermissionsMessage);
				delete this._pendingPermissions;
				this._loadingPermissions = 0;
				this._loadPermissionsErrorCallback(error);
			}
		},
		
		/**
		 * Loads the set of permissions for all roles for a particular time period
		 * from the server, if necessary. If data is successfully retrieved from
		 * the or it has already been loaded, the provided function successCallback
		 * is invoked with no arguments.
		 * 
		 * @param	contextItem		Item in the process area store for the time
		 * 							period to load permissions for
		 * @param	successCallback	function
		 * @param	errorCallback	function( error )                                           be displayed to the user
		 */
		loadPermissionsForAllRoles: function(contextItem, successCallback, errorCallback) {
			if (this._permissionExtensionsStore === undefined) {
				throw new Error("Before loading permissions, permission extension data must be loaded via function loadPermissionExtensions()"); //$NON-NLS-1$
			}
			
			var needToLoad = false;
			var allRoles = this._rolesStore.getAllRoleItems();
			for (var i = 0; i < allRoles.length && !needToLoad; i++) {
				var roleId = this._rolesStore.getValue(allRoles[i], ProcessConstants.ProcessRoleDTO.ID);
				var storeId = this._getStoreId(roleId, contextItem);
				if (!this._loadedPermissions[storeId]) {
					needToLoad = true;
				}
			}
			
			if (needToLoad) {
				if (this._pendingPermissionsForAllRoles) {
					throw new Error(dojo.string.substitute("Already loading permissions for ${0}", [JSON.stringify(this._pendingPermissionsForAllRoles)])); //$NON-NLS-1$
				}
				
				this._loadPermissionsSuccessCallback = successCallback;
				this._loadPermissionsErrorCallback = errorCallback;
				
				this._pendingPermissionsForAllRoles = {
					contextItem: contextItem
				};
				
				this._loadingPermissionsMessage = this.messages.PermissionsWorkingCopy_3;
				this._startLoadingFunction(this._loadingPermissionsMessage);
				
				var fetchPermissionsForAllRoles = dojo.hitch(this, function(context, contextId) {
					this._getPermissionsForAllRolesFromServer(context, contextId, "_handlePermissionsForAllRoles", "_handlePermissionsForAllRolesFetchError"); //$NON-NLS-1$ //$NON-NLS-2$					
				});
				this._loadingPermissionsForAllRoles += 1;
				if (contextItem === this._processAreaStore.alwaysRootItem) {			
					if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
						this._loadingPermissionsForAllRoles += 1;
						fetchPermissionsForAllRoles(ProcessConstants.PermissionContext.PROJECT, ""); //$NON-NLS-1$
					}
					fetchPermissionsForAllRoles(ProcessConstants.PermissionContext.TEAM, ""); //$NON-NLS-1$
				} else {
					fetchPermissionsForAllRoles(this._processAreaStore.getValue(contextItem, "enabledContext"), this._processAreaStore.getSavedContextId(contextItem)); //$NON-NLS-1$
				}
			} else {
				setTimeout(function() {
					successCallback();
				});
			}
		},
			
		_getPermissionsForAllRolesFromServer: function(context, contextId, successCallback, errorCallback ) {
			var srh = new ServiceResponseHandler(this, successCallback, errorCallback); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getPermissionsForAllRoles({
				context: context,
				contextId: contextId,
				owningApplicationKey: this._owningApplicationKey,
				processAreaContext: this._editorContext,
				processAreaItemId: this._processAreaItemId
			}, srh);	
		},
		
		_handlePermissionsForAllRoles: function(data) {
			if (this._loadingPermissionsForAllRoles !== 0) {
				var rolesWithPermissions = {};
				if (data==null) {
					data = [];
				}
				
				// data is an array containing permissions data for those roles which have any permissions
				// Start by determining which roles have permissions defined in data				
				dojo.forEach(data, dojo.hitch(this, function(permissionsForRole) {
					rolesWithPermissions[permissionsForRole.roleId] = true;
				}));

				// Add empty arrays of permissions for any that were not in the  
				// response so stores will be created for those roles containing
				// 'not permitted'/'inherited' for all actions
				dojo.forEach(this._rolesStore.getAllRoleItems(), dojo.hitch(this, function(role) {
					var roleId = this._rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.ID);
					if (!rolesWithPermissions[roleId]) {
						data.push({roleId: roleId, permissions: []});
					}
				}));
				
				/*
				 * processes one role from data and calls itself with using setTimeout to process
				 * the remaining roles. Using setTimeout prevents getting a dialog from
				 * Internet Explorer about a busy script
				 */
				var processPermissions = dojo.hitch(this, function() {
					if (data.length > 0) {
					    var permissionsForRole = data.shift();
					    var storeId = this._getStoreId(permissionsForRole.roleId, this._pendingPermissionsForAllRoles.contextItem);
						//Skip this role if the permissions have already been loaded
						if (!this._loadedPermissions[storeId]) {
							this._populatePermissions(permissionsForRole.roleId, permissionsForRole.permissions, this._pendingPermissionsForAllRoles, this._loadingPermissionsForAllRoles);
						} 
					    setTimeout(processPermissions);
					} else {
						this._currentlyProcessingPermissionsForAllRoles = undefined;
						if (this._loadingPermissionsForAllRoles === 0) {
							this._stopLoadingFunction(this._loadingPermissionsMessage);
							delete this._pendingPermissionsForAllRoles;
							this._loadPermissionsSuccessCallback();
						}
					}
				});
				/*
				 * When permissions are fetched for a project area two server calls
				 * are made, one to fetch project permissions and one to fetch team permissions.
				 * Complete processing the first set of permissions before starting to process 
				 * the second set so that _populatePermissions will get called with 
				 * the correct value for numLoading
				 */
				var processPermissionsWhenNotBusy = dojo.hitch(this, function() {
					if (this._currentlyProcessingPermissionsForAllRoles) {
						setTimeout(processPermissionsWhenNotBusy, 50);
					} else {
					    this._loadingPermissionsForAllRoles -= 1;
					    this._currentlyProcessingPermissionsForAllRoles = true;
					    processPermissions();
				    }
				});
				
				processPermissionsWhenNotBusy();
			}
		},
		
		_handlePermissionsForAllRolesFetchError: function(error) {
			if (this._loadingPermissionsForAllRoles !== 0) {
				this._stopLoadingFunction(this._loadingPermissionsMessage);
				delete this._pendingPermissionsForAllRoles;
				this._loadingPermissionsForAllRoles = 0;
				this._loadPermissionsErrorCallback(error);
			}
		},
		
		/*
		 * Find an existing store for this role and context or create a new one
		 * and populate the store with extension data
		 */
		_populatePermissions: function(roleId, data, loadArgs, numLoading) {
			var storeId = this._getStoreId(roleId, loadArgs.contextItem);
			var store = this._permissionsStores[storeId];
			if (!store) {
				store = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore({
					permissionExtensionsStore: this._permissionExtensionsStore,
					isChildArea: this._isChildArea,
					processAreaStore: this._processAreaStore,
					pauseNotifications: dojo.hitch(this, this._pauseNotifications)
				});
				this._permissionsStores[storeId] = store;
				this._permissionsStoreInfos[storeId] = {
					roleId: roleId,
					contextItem: loadArgs.contextItem
				};
			}
			this._populatePermissionStore(store, data, numLoading);
			
			//Save the store so that we can detect which items are dirty
			store.save();
				
			if (numLoading == 0) {
				this._loadedPermissions[storeId] = true;
				
				//Connect to onSet events to notify listeners when permissions
				//or properties are changed from now on
				this._storeConnects.push(dojo.connect(store, "onSet", dojo.hitch(this, function(item, attribute, oldValue, newValue) { //$NON-NLS-1$
					this._permissionChanged(store, item, roleId, loadArgs.contextItem);
				})));
			}
		},
		
		/*
		 * Populate a permission store with extension data. This is used for 
		 * loading permissions for this process area as well as for loading
		 * inherited permissions when customizing.
		 * @param numLoading represents the number of server calls outstanding
		 * that are reading data for this store
		 */
		_populatePermissionStore: function(store, data, numLoading) {
			//Calculate the 'permission' state for every category, operation,
			//and action
			
			//First, handle each item that is permitted based on the response
			//from the /permissions service. For each:
			//1. Mark the item as 'permitted'
			//2. Mark all child items as 'permitted' recursively
			//3. Mark each parent item as 'some permitted'
			dojo.forEach(data, dojo.hitch(this, function(permissionObj) {
				var permitItemAndChildren = function(item) {
					store.setValue(item, PROP_PERMISSION, PermittedType.PERMITTED);
					
					dojo.forEach(store.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(childItem) {
						permitItemAndChildren(childItem);
					});
				};
				var segments = permissionObj.id.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
				store.ensureOperationInStore(segments[0], null);
				store.fetch({
					query: {
						id: "*" + permissionObj.id //$NON-NLS-1$
					},
					queryOptions: {
						deep: true
					},
					onItem: function (item) {
						if (permissionObj.enabled === true) {
							permitItemAndChildren(item);
							
							var parentId = store.getValue(item, PROP_PARENT_ID);
							while (parentId !== undefined) {
								store.fetchItemByIdentity({
									identity: parentId,
									onItem: function(operationItem) {
										if (operationItem === null) {
											parentId = undefined;
										} else {
											var permissionForParent = PermittedType.PERMITTED;
											var children = store.getValues(operationItem, ProcessConstants.ExtensionDTO.ITEMS);
											for (var i = 0; i < children.length && permissionForParent === PermittedType.PERMITTED; i++) {
												if (PermittedType.PERMITTED !== store.getValue(children[i], PROP_PERMISSION)) {
													permissionForParent = PermittedType.SOME_PERMITTED;
												}
											}
											store.setValue(operationItem, PROP_PERMISSION, permissionForParent);
											parentId = store.getValue(operationItem, PROP_PARENT_ID);
										}
									}
								});
							}
						}
						
						// Only operations support the final attribute
						if (store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP)) {
							store.setValue(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP, permissionObj.finalOp);
						}
						
						// Only operations support the denial explanation attribute
						if (store.hasAttribute(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION)) {
							store.setValue(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION, permissionObj.denialExplanation);
						}
					}
				});
			}));
			
			if (numLoading === 0) {				
				//Now that all requests to get permissions have returned,
				//iterate over each operation. For operations that are 'some permitted', 
				//mark every child who is not permitted to 'not permitted'. For operations 
				//that have not been touched, mark as 'not permitted'.
				store.fetch({
					onItem: dojo.hitch(this, function(operationItem) {						
						var permission = store.getValue(operationItem, PROP_PERMISSION);
						var permissionForChildren;
						if (PermittedType.SOME_PERMITTED === permission) {
							permissionForChildren = PermittedType.NOT_PERMITTED;
						} else if (undefined === permission) {
							permissionForChildren = PermittedType.NOT_PERMITTED;
							store.setValue(operationItem, PROP_PERMISSION, permissionForChildren);
						}

						if (permissionForChildren !== undefined) {
							var setPermissionOfChildren = function(item, permission) {
								dojo.forEach(store.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(childItem) {
									if (store.getValue(childItem, PROP_PERMISSION) === undefined) {
										store.setValue(childItem, PROP_PERMISSION, permission);
									}
									setPermissionOfChildren(childItem, permission);
								});
							};
							setPermissionOfChildren(operationItem, permissionForChildren);
						}
					})
				});
			}
		},
		
		_getStoreId: function(roleId, contextItem) {
			var storeId = ""; //$NON-NLS-1$
			if (roleId !== null) {
				storeId += roleId;
				storeId += ProcessConstants.PERMISSION_PATH_DELIMITER;
			}
			if (contextItem !== this._processAreaStore.alwaysRootItem) {
				storeId += this._processAreaStore.getValue(contextItem, "id"); //$NON-NLS-1$
			}
			return storeId;
		},
		
		/**
		 * Return a dojo.data store for a combination of role and time period.
		 * This store is required to use helper functions such as getXXX() to
		 * retrieve information about an item.
		 * 
		 * If the permissions for the combination of role and time period have
		 * not been loaded via a call to loadPermissions(), null is returned.
		 */
		getPermissionsStore: function(roleId, contextItem) {
			var store = this._permissionsStores[this._getStoreId(roleId, contextItem)];
			return store || null;
		},
		
		/**
		 * Given the id of a category, operation or action determine whether it is
		 * the id of an operation.
		 * 
		 * @param operationId    Identifier of a category, operation or action
		 * 
		 * @return true if the identifier is the identifier of an operation
		 */
		isOperation: function(operationId) {
			var segments = operationId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			return segments.length == 2;
		},
		
		/**
		 * Given an item from the extensions store, determine whether it is a
		 * project operation.
		 * 
		 * @param operationItem    An item in the permission extensions store which
		 * represents a category, operation or action
		 * 
		 * @return true if the identifier is the identifier of an operation
		 */
		isProjectOperation: function(operationItem) {
			return ProcessConstants.ExtensionContext.PROJECT === this._permissionExtensionsStore.getValue(operationItem, ProcessConstants.ExtensionDTO.CONTEXT);
		},
		
		/**
		 * Given the id of a category, operation or action determine whether it is
		 * the id of an category.
		 * 
		 * @param operationId    Identifier of a category, operation or action
		 * 
		 * @return true if the identifier is the identifier of a category
		 */
		isCategory: function(operationId) {
			var segments = operationId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			return segments.length == 1;
		},
		
		/**
		 * Given an item from the extensions store, determine whether it is a
		 * category with no team operations
		 * 
		 * @param operationItem    An item in the permission extensions store which
		 * represents a category, operation or action
		 * 
		 * @return true if the identifier is the identifier of a category containing 
		 * only project operations
		 */
		isCategoryWithOnlyProjectOperations: function(operationItem) {
			return true === this._permissionExtensionsStore.getValue(operationItem, PROP_HAS_ONLY_PROJECT_OPERATIONS);
		},
		
		/**
		 * Get the id of the operation which is the parent or ancestor of an action
		 * @param actionId    Identifier of an operation or action
		 * 
		 * @return if actionId is the id of an action return the parent operation id, otherwise 
		 * return actionId
		 */
		getParentOperationId: function(actionId) {
			var segments = actionId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			if (segments.length <= 2) {
				return actionId;
			} 
			return segments[0] + ProcessConstants.PERMISSION_PATH_SEPARATOR + segments[1];
		},
		
		/**
		 * Used to determine whether permissions are inherited in the "always" timeframe
		 * 
		 * @return true if the underlying process area is a team area or a project area which
		 * inherits its process from another project area.
		 */
		isChildArea: function() {
			return this._isChildArea;
		},
		
		/**
		 * Get the name of an operation or action.
		 * 
		 * @param actionId    Identifier of an operation or action
		 * 
		 * @return	The name of the operation if permission extensions have been
		 * 			loaded; otherwise an exception is thrown.
		 */
		getName: function(actionId) {
			if (this._permissionExtensionsStore) {
				var name;
				this._permissionExtensionsStore.fetchItemByIdentity({
					identity: actionId,
					onItem: dojo.hitch(this, function(item) {
						if (item !== null) {
							name = this._permissionExtensionsStore.getValue(item, ProcessConstants.ExtensionDTO.NAME);
						}
					})
				});
				if (name === undefined) {
					throw new Error(dojo.string.substitute("Internal error; name for ${0} is unknown", [actionId])); //$NON-NLS-1$
				}
				return name;
			}
			throw new Error("Permission extensions have not been loaded"); //$NON-NLS-1$
		},
		
		/**
		 * Clients that wish to be notified when a permission or operation
		 * property changes can dojo.connect() to this function.
		 * 
		 * @param	store		Dojo.data store for the permission that changed
		 * @param	item		Dojo.data item for the permission that changed
		 * @param	roleId		The identifier of the role for which permission
		 * 						has changed
		 * @param	context		The context for which permission has changed
		 * @param	contextId	The context identifier for which permission has
		 * 						changed
		 */
		permissionChanged: function(store, item, roleId, contextItem) {
		},
		
		_permissionChanged: function(store, item, roleId, contextItem) {
			//Update maps _configuredRolesByContextId and
			//_configuredScopesByRoleId that maintain the
			//state of customized roles and scopes
			var contextId = this._processAreaStore.getSavedContextId(contextItem);
			
			var configuredRoles = this._configuredRolesByContextId[contextId];
			if (configuredRoles === undefined) {
				configuredRoles = {};
				this._configuredRolesByContextId[contextId] = configuredRoles;
			}
			
			var configuredScopes = this._configuredScopesByRoleId[roleId];
			if (configuredScopes === undefined) {
				configuredScopes = {};
				this._configuredScopesByRoleId[roleId] = configuredScopes;
			}
			
			if (PermittedType.INHERITED === store.getPermission(item, contextItem)) {
				var isCustomized = false;
				store.fetch({
					queryOptions: {
						deep: true
					},
					onItem: function(item, request) {
						if (store.getValue(item, PROP_PERMISSION) !== PermittedType.INHERITED) {
							isCustomized = true;
							request.abort();
						}
					}
				});
				if (!isCustomized) {
					delete configuredRoles[roleId];
					delete configuredScopes[contextId];
				}
			} else {
				configuredRoles[roleId] = true;
				configuredScopes[contextId] = contextItem;
			}
			
			for (var i = 0; i < this._pausedNotificationStores.length; i++) {
				if (this._pausedNotificationStores[i] === store) {
					return;
				}
			}
			this.permissionChanged(store, item, roleId, contextItem);
		},
		
		_pauseNotifications: function(store, pause) {
			if (pause) {
				this._pausedNotificationStores.push(store);
			} else {
				for (var i = 0; i < this._pausedNotificationStores.length; i++) {
					if (this._pausedNotificationStores[i] === store) {
						this._pausedNotificationStores.splice(i, 1);
						break;
					}
				}
			}
		},
		
		/**
		 * Invoked by clients that are about to initiate a number of updates
		 * to stores managed by the working copy. The caller is responsible
		 * for calling endBatchUpdate() when finished.
		 * 
		 * Other clients may optionally dojo.connect to this function and
		 * endBatchUpdate, in combination with permissionChanged, to optimize
		 * reaction to changes when possible.
		 */
		startBatchUpdate: function() {},
		
		/**
		 * See startBatchUpdate().
		 */
		endBatchUpdate: function() {},
		
		/**
		 * Returns whether there are pending changes to apply on editor save.
		 */
		changesPending: function() {
			for (var storeKey in this._permissionsStores) {
				if (this._permissionsStores[storeKey].isDirty()) {
					return true;
				}
			}
			return false;
		},
		
		/**
		 * Called after data has been saved
		 */
		saved: function(saveResponse) {
			for (var storeKey in this._permissionsStores) {
				this._permissionsStores[storeKey].save();
			}
		},
		
		aboutToBeSaved: function(saveObj) {
			/**
			 * Helper to access a field in an object and create it if it does
			 * not already exist.
			 */
			var getObjectField = function(parent, name) {
				if (parent[name] === undefined) {
					parent[name] = {};
				}
				return parent[name];
			};
			
			/**
			 * Helper to access a field in an object and create it if it does
			 * not already exist.
			 */
			var getArrayField = function(parent, name) {
				if (parent[name] === undefined) {
					parent[name] = [];
				}
				return parent[name];
			};
			
			/**
			 * Helper to set a boolean value for a field in an object,
			 * defaulting to false if the value is null or undefined.
			 */
			var setBooleanProperty = function(object, propName, value) {
				if (value === undefined || value == null) {
					value = false;
				}
				object[propName] = value; //$NON-NLS-1$
			};
			
			/**
			 * Helper to set a string value for a field in an object,
			 * defaulting to "" if the value is null or undefined.
			 */
			var setTextProperty = function(object, propName, value) {
				if (value === undefined || value == null) {
					value = ""; //$NON-NLS-1$
				}
				object[propName] = value; //$NON-NLS-1$
			};
			
			/**
			 * Helper to fill in the actionPaths field.
			 *  
			 * Construct an array of all child items whose permission matches
			 * a given value.
			 */
			var addActionPaths = function(actionPathsArray, operationPath, store, item, permission) {
				if (store.isDirty(item) && permission === store.getValue(item, PROP_PERMISSION)) {
					var itemPath = store.getValue(item, ProcessConstants.ExtensionDTO.ID);
					actionPathsArray.push(itemPath.substring(operationPath.length));
				}
				dojo.forEach(store.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(childAction) {
					addActionPaths(actionPathsArray, operationPath, store, childAction, permission);
				});
			};
			
			var permissionContainers = {};
			
			for (var storeKey in this._permissionsStores) {
				var store = this._permissionsStores[storeKey];
				if (store.isDirty()) {
					var storeInfo = this._permissionsStoreInfos[storeKey];
					
					//Ensure that the context and role are valid, eg. that either
					//hasn't been deleted
					if (!this._isValidContext(storeInfo.contextItem) || !this._isValidRole(storeInfo.roleId)) {
						continue;
					}
					
					var self = this;
					
					//Iterate over top-level items, which should represent
					//operations
					store.fetch({
						query: {
							id: "*" //$NON-NLS-1$
						},
						onItem: function(operationItem) {
							if (store.isDirty(operationItem)) {
								//First, determine which container to use,
								//(jsonIterationPermissions, jsonIterationPermissions,
								//etc...) and how to create the structure above
								//the map of roles to operation ids
								var permissionContainerName, getRolesToOperations;
								
								if (self._processAreaStore.alwaysRootItem == storeInfo.contextItem && ProcessConstants.ExtensionContext.PROJECT === store.getValue(operationItem, ProcessConstants.ExtensionDTO.CONTEXT)) {
									//Project operation
									permissionContainerName = "jsonProjectConfigurationPermissions"; //$NON-NLS-1$
									getRolesToOperationsField = function(rolesToOperationsParent) {
										return getObjectField(rolesToOperationsParent, "rolesToOperations"); //$NON-NLS-1$
									};
								} else if (self._processAreaStore.alwaysRootItem == storeInfo.contextItem && ProcessConstants.ExtensionContext.TEAM === store.getValue(operationItem, ProcessConstants.ExtensionDTO.CONTEXT)) {
									//Team operation
									permissionContainerName = "jsonTeamConfigurationPermissions"; //$NON-NLS-1$
									getRolesToOperationsField = function(rolesToOperationsParent) {
										return getObjectField(rolesToOperationsParent, "rolesToOperations"); //$NON-NLS-1$
									};
								} else {
									var context = self._processAreaStore.getValue(storeInfo.contextItem, "enabledContext"); //$NON-NLS-1$
									var contextId = self._processAreaStore.getValue(storeInfo.contextItem, "contextId"); //$NON-NLS-1$
									
									if (ProcessConstants.PermissionContext.DEV_LINE == context) {
										//Development line
										permissionContainerName = "jsonDevelopmentLinePermissions"; //$NON-NLS-1$
										getRolesToOperationsField = function(rolesToOperationsParent) {
											var idToRolesToOperations = getObjectField(rolesToOperationsParent, "idToRolesToOperations"); //$NON-NLS-1$
											return getObjectField(idToRolesToOperations, contextId);
										};
									} else if (ProcessConstants.PermissionContext.ITERATION == context) {
										//Iteration
										permissionContainerName = "jsonIterationPermissions"; //$NON-NLS-1$
										getRolesToOperationsField = function(rolesToOperationsParent) {
											var fullPath = contextId;
											var devLineId, iterationPath;
											var segments = fullPath.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
											if (segments.length > 1) {
												var devLineId = null;
												if (self._editorContext === ProcessConstants.EditorContext.PROJECT) {
													devLineId = segments.shift();
												} else {
													devLineId = segments[0];
												}
												iterationPath = segments.join(ProcessConstants.PERMISSION_PATH_SEPARATOR);
											} else {
												throw new Error(dojo.string.substitute("Invalid context id for iteration: ${0}", [contextId])); //$NON-NLS-1$
											}
											
											var developmentLineToIterationPaths = getObjectField(rolesToOperationsParent, "developmentLineToIterationPaths"); //$NON-NLS-1$
											var developmentLine = getObjectField(developmentLineToIterationPaths, devLineId);
											return getObjectField(developmentLine, iterationPath);
										};
									} else if (ProcessConstants.PermissionContext.ITERATION_TYPE === context) {
										//Iteration type
										permissionContainerName = "jsonIterationTypePermissions"; //$NON-NLS-1$
										getRolesToOperationsField = function(rolesToOperationsParent) {
											var idToRolesToOperations = getObjectField(rolesToOperationsParent, "idToRolesToOperations"); //$NON-NLS-1$
											return getObjectField(idToRolesToOperations, contextId);
										};
									}
								}
								
								//Then, fill in the mapping of role to operation
								
								//Helper to create the tree starting at root
								//"granted" or "revoked"
								var addPermissionConfiguration = function(parentFieldName, actionPathPermission, overrideCleared) {
									var parentField = getObjectField(getObjectField(permissionContainers, permissionContainerName), parentFieldName);

									var rolesToOperationsField = getRolesToOperationsField(parentField);
									
									var operationPath = store.getValue(operationItem, ProcessConstants.ExtensionDTO.ID);
									var segments = operationPath.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
									var operationId = segments[1];

									var roleObj = getObjectField(rolesToOperationsField, storeInfo.roleId);
									var operationObj = getArrayField(roleObj, operationId);
									var configurationObj = {};
									operationObj.push(configurationObj);
									
									if (overrideCleared === true) {
										setBooleanProperty(configurationObj, SAVE_FIELD_OVERRIDE_CLEARED, true);
									}
									
									setBooleanProperty(configurationObj, SAVE_FIELD_FINAL, store.getValue(operationItem, ProcessConstants.PermissionConfigurationDTO.FINAL_OP)); //$NON-NLS-1$
									
									var explanationConfigured = store.getValue(operationItem, PROP_DENIAL_EXPLANATION_CONFIGURED);
									if (false === explanationConfigured) {
										setTextProperty(configurationObj, SAVE_FIELD_EXPLANATION, null);
									} else {
										setTextProperty(configurationObj, SAVE_FIELD_EXPLANATION, store.getValue(operationItem, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION)); //$NON-NLS-1$
									}
									
									var actionPaths = getArrayField(configurationObj, SAVE_FIELD_ACTION_PATHS);
									addActionPaths(actionPaths, operationPath, store, operationItem, actionPathPermission);
								};
								
								var permission = store.getValue(operationItem, PROP_PERMISSION);
								if (PermittedType.PERMITTED === permission) {
									addPermissionConfiguration(SAVE_FIELD_GRANTED, permission);
								} else if (PermittedType.NOT_PERMITTED === permission) {
									addPermissionConfiguration(SAVE_FIELD_REVOKED, permission);
								} else if (PermittedType.SOME_PERMITTED === permission) {
									addPermissionConfiguration(SAVE_FIELD_GRANTED, PermittedType.PERMITTED);
									addPermissionConfiguration(SAVE_FIELD_REVOKED, PermittedType.NOT_PERMITTED);
								} else if (PermittedType.INHERITED === permission) {
									addPermissionConfiguration(SAVE_FIELD_REVOKED, PermittedType.INHERITED, true);
								}
							}
						}
					});
				}
			}
			
			//Attach any JSON that was created to the save object
			for (var permissionContainerName in permissionContainers) {
				getArrayField(saveObj, permissionContainerName).push(dojo.toJson(permissionContainers[permissionContainerName])); //$NON-NLS-1$
			}
		},
		
		/**
		 * Returns whether the combination of context and contextId are
		 * currently valid; a typical case where a combination would be invalid
		 * would be if a newly created timeline, iteration, or iteration type
		 * had been deleted.
		 */
		_isValidContext: function(contextItem, contextId) {
			return this._processAreaStore.isItem(contextItem);
		},
		
		/**
		 * Returns whether a role identifier is currently valid; a typical case
		 * where a combination would be invalid would be if it had been deleted.
		 */
		_isValidRole: function(roleId) {
			return null !== this._rolesStore.fetchRoleById(roleId);
		},
		
		/**
		 * Initialize the mapping of roles to configured permissions scopes
		 * that is used to answer which scopes are configured for a given role
		 * and which roles have configured permissions for a given scope.
		 * 
		 * This function loads data from the server asynchronously. If data is
		 * loaded successfully, successCallback will be invoked. No arguments
		 * are passed to the callback; invoking it is an indication that
		 * functions permissionsAreConfiguredForSpecificScope(),
		 * getConfiguredRoles(), and getConfiguredScopes() can be called.
		 * 
		 * If loading fails, errorCallback is invoked with the error from the
		 * server.
		 * 
		 * @param	successCallback	function
		 * @param	errorCallback	function
		 */
		loadConfiguredScopes: function(successCallback, errorCallback) {
			if (this._configuredScopes === undefined) {
				this._loadingConfiguredScopesSuccessCallback = successCallback;
				this._loadingConfiguredScopesErrorCallback = errorCallback;
				
				this._startLoadingFunction(this.messages.PermissionsWorkingCopy_4);
				
				this._getConfiguredScopesFromServer(this._processAreaItemId, "_populateConfiguredScopes", "_handleConfiguredScopesFetchError"); //$NON-NLS-2$ //$NON-NLS-1$
				
			} else {
				setTimeout(dojo.hitch(this, function() {
					successCallback();
				}));
			}
		},
			
		_getConfiguredScopesFromServer: function(processAreaItemId, successCallback, errorCallback) {
			var srh = new ServiceResponseHandler(this, successCallback, errorCallback); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getConfiguredPermissionsScopes({
				processAreaContext: this._editorContext,
				processAreaItemId: processAreaItemId
			}, srh);
		},
		
		_populateConfiguredScopes: function(data) {
			if (!this._processAreaStore.isInitialized()) {
				// contexts need to be loaded to continue
				setTimeout(dojo.hitch(this, function() {
					this._populateConfiguredScopes(data);
				}), 100);
				return;
			}
			
			if (data === null) {
				data = [];
			}
			
			this._configuredScopes = true;
			this._configuredRolesByContextId = {};
			this._configuredScopesByRoleId = {};
			
			for (var i = 0; i < data.length; i++) {
				var roleId = data[i].roleId;
				
				//Ignore roles whose id no longer matches a role in the store
				if (!this._isValidRole(roleId)) {
					continue;
				}
				
				var permissionScopes = data[i].permissionScopes;
				
				for (var j = 0; j < permissionScopes.length; j++) {
					
					var processScopeItem = dojo.hitch(this, function(scopeItem) {
						var contextId = permissionScopes[j].contextId || ""; //$NON-NLS-1$
							
						var configuredRoles = this._configuredRolesByContextId[contextId];
						if (configuredRoles === undefined) {
							configuredRoles = {};
							this._configuredRolesByContextId[contextId] = configuredRoles;
						}
						configuredRoles[roleId] = true;
						
						var configuredScopes = this._configuredScopesByRoleId[roleId];
						if (configuredScopes === undefined) {
							configuredScopes = {};
							this._configuredScopesByRoleId[roleId] = configuredScopes;
						}
						configuredScopes[contextId] = scopeItem;
					});
					
					if (permissionScopes[j].contextId === null) {
						processScopeItem(this._processAreaStore.alwaysRootItem);
					} else {
						var query = {
						};
						query.enabledContext = permissionScopes[j].context;
						if (ProcessConstants.PermissionContext.ITERATION === permissionScopes[j].context) {
							query.originalContextId = permissionScopes[j].contextId;
						} else {
							query.contextId = permissionScopes[j].contextId;
						}
						this._processAreaStore.fetch({
							query: query,
							queryOptions: {
								deep: permissionScopes[j].contextId !== null
							},
							onItem: dojo.hitch(this, function(scopeItem) {
								processScopeItem(scopeItem);
							})
						});
					}
				}
			}
			
			this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_4);
			this._loadingConfiguredScopesSuccessCallback();
		},
		
		_handleConfiguredScopesFetchError: function(error) {
			this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_4);
			this._loadingConfiguredScopesErrorCallback(error);
		},
		
		/**
		 * Return whether permissions are configured for any time period
		 * besides Always.
		 * 
		 * Before calling this, loadConfiguredScopes() must be called and
		 * its success callback invoked.
		 */
		permissionsAreConfiguredForSpecificScope: function() {
			for (var contextId in this._configuredRolesByContextId) {
				if (contextId !== null && 0 !== contextId.length) {
					for (var roleId in this._configuredRolesByContextId[contextId]) {
						//Return true if any roles are configured
						return true;
					}
				}
			}
			return false;
		},
		
		/**
		 * Returns a map whose keys are role identifiers representing roles for
		 * which permissions are configured for a given time period.
		 * 
		 * The value for each key is 'true', (to allow constant time lookup of
		 * whether a role is configured). Roles for which permissions are not
		 * configured are not represented in the map.
		 * 
		 * @param	contextItem
		 */
		getConfiguredRoles: function(contextItem) {
			var contextId = this._processAreaStore.getSavedContextId(contextItem);
			return this._configuredRolesByContextId[contextId] || {};
		},
		
		/**
		 * Returns a map whose keys are context identifiers representing scopes
		 * for which permissions are configured for a role identifier.
		 * 
		 * The value for each key is the item for that scope in the process
		 * area store. Scopes for which permissions are not configured are not
		 * represented in the map.
		 * 
		 * If the argument is null, the union of configured scopes across all
		 * roles is returned.
		 * 
		 * @param	contextItem
		 */
		getConfiguredScopes: function(roleId) {
			var result;
			if (roleId === null) {
				result = {};
				for (roleId in this._configuredScopesByRoleId) {
					for (var contextId in this._configuredScopesByRoleId[roleId]) {
						result[contextId] = this._configuredScopesByRoleId[roleId][contextId];
					}
				}
			} else {
				result = this._configuredScopesByRoleId[roleId] || {};
			}
			if (!this._isChildArea) {
				result[""] = this._processAreaStore.alwaysRootItem; //$NON-NLS-1$
			}
			return result;
		},
		
		/**
		 * Update whether the process area whose permissions are tracked is
		 * a process consumer. This has an effect on the default value for
		 * permissions that are not specified.
		 * 
		 * @param	isChildArea	boolean 
		 */
		setIsChildArea: function(isChildArea) {
			this._isChildArea = isChildArea;
		},
		
		/**
		 * Update the process area store.
		 */
		setProcessAreaStore: function(processAreaStore) {
			this._processAreaStore = processAreaStore;
		},
		
		/**
		 * Fetch or compute, (if locally configured), the permissions for an
		 * operation that are defined in the nearest ancestor scope.
		 * 
		 * If permissions are successfully calculated, the callback
		 * successCallback is invoked, passing an object of the following
		 * format:
		 * 
		 * {
		 * 		context				Type of winning scope; one of
		 * 							ProcessConstants.PermissionContext
		 * 		contextTitle		Title of winning scope, (name of project,
		 * 							team, iteration, etc...)
		 * 		areaContext			Type of enclosing process area of winning
		 * 							scope; one of
		 * 							ProcessConstants.PermissionContext.PROJECT or
		 * 							ProcessConstants.PermissionContext.TEAM 
		 * 		areaContextTitle	Title of enclosing process area of winning
		 * 							scope
		 * 		getPermission		Function that takes an operation or action
		 * 							identifier and returns the permission in the
		 * 							winning scope
		 * 		isFinal				Whether the configuration is final in a
		 * 							parent scope
		 * 		denialExplanation	Denial explanation in the winning scope
		 * }
		 * 
		 * If there is an error in a request to the server that is required for
		 * computation, the callback errorCallback is invoked, passing the
		 * error object for the request. 
		 */
		getInheritedPermissions: function(roleId, contextItem, operationId, successCallback, errorCallback) {
			// There are up to four steps required to get inherited permissions:
			// 1: determine all the scopes which permissions could be inherited from
			// 2: without considering parent process areas determine in which of those 
			// scopes permissions have been declared and whether they are final.
			// If the permissions are final in a parent scope stop here.
			// 3: if this is a team area or project area which inherits process call the
			// server to determine which scope the permissions will be inherited from
			// 4: if the permissions are inherited from the parent process area load them 
			this._startLoadingFunction(this.messages.PermissionsWorkingCopy_5);
			
			//
			// activeScopes is an array of all of the scopes that the permissions could 
			// possibly inherit from, for each scope it contains the contextItem, and
			// two booleans which specify whether permission was defined in that scope and 
			// whether it was final. These booleans are left undefined until we have determined
			// their correct value.
			//
			var activeScopes = [{
				contextItem : contextItem,
				permissionDefined : false,
			    permissionFinal : false
			}];
			var context = this._processAreaStore.getValue(contextItem, "enabledContext"); //$NON-NLS-1$
			// For an iteration add the iteration type, and all parent
			// iterations and their iteration type as well as the timeline
			if (context === ProcessConstants.PermissionContext.ITERATION) {
				// scopeItem will be used to examine each parent scope in a loop
				var scopeItem = contextItem;
				var scopeContext = context;
				// The current scope has already been pushed - process its iteration type
				// and loop through the parent scopes as far as the dev line. 
				while (scopeContext === ProcessConstants.PermissionContext.ITERATION) {
					var iterationTypeId = this._processAreaStore.getValue(scopeItem, "iterationTypeItemId"); //$NON-NLS-1$
					if (iterationTypeId !== ProcessConstants.ITERATION_TYPE_NONE) {
						this._processAreaStore.fetchItemByIdentity({
							identity: iterationTypeId,
							onItem: dojo.hitch(this, function(item, request) {
								activeScopes.push({
									contextItem : item
								});							
							})
						});
					}
					var parentIdentity = this._processAreaStore.getValue(scopeItem, "parent"); //$NON-NLS-1$
					scopeItem = null; 
					scopeContext = null;
					this._processAreaStore.fetchItemByIdentity({
						identity : parentIdentity,
						onItem: dojo.hitch(this, function(parentItem) {
							scopeItem = parentItem;
							scopeContext = this._processAreaStore.getValue(scopeItem, "enabledContext"); //$NON-NLS-1$
						})
					});
					
					var scopeData = {
						contextItem : scopeItem
					};
					// Dev line permissions will never be defined in a team area but the dev line
					// still needs to be passed to the server call to compute inherited permissions
					if (scopeContext === ProcessConstants.PermissionContext.DEV_LINE && this._editorContext === ProcessConstants.EditorContext.TEAM) {
					    scopeData.permissionDefined = false;
					    scopeData.permissionFinal = false;
					}
					activeScopes.push(scopeData);
				}
			}
			// Add the always scope unless we are in the always scope in which case it has already been added
			if (contextItem !== this._processAreaStore.alwaysRootItem) {
				activeScopes.push({
					contextItem : this._processAreaStore.alwaysRootItem
				});
			}
			
			// Walk through the permission scope arrays from least qualified to 
			// most qualified and fill in the configuration state
			// The inner functions that follow are chained to create the effect of a loop 
			// 
			var isConfigured = false;
			var isFinalInParent = false;
			// WinningContextItem and WinningStore represent the scope
			// that ends up defining the inherited permissions
			var winningStore = undefined;
			var winningContextItem = undefined;
			var nextScopeIndex = activeScopes.length - 1;
			
			/*
			 * function which reads the permission state in the parent scopes 
			 * in this editor from least qualified to most qualified, loading
			 * permissions as necessary
			 */
			var checkEditorPermissionState = dojo.hitch(this, function() {			
				if (activeScopes[nextScopeIndex].permissionDefined === undefined) {
					this.loadPermissions(roleId, activeScopes[nextScopeIndex].contextItem, updateActiveScope, errorCallback);
				} else {
					completeProcessingCurrentScope();
				}
			});
			
			/*
			 * function called after permissions have been loaded for the next context
			 * As soon as a final parent scope is found no more scopes are processed so it
			 * is safe to assume that isFinalInParent is false on entry to this function
			 */
			var updateActiveScope = dojo.hitch(this, function() {	
				var currentActiveScope = activeScopes[nextScopeIndex];
				var currentContextItem = currentActiveScope.contextItem;
				var currentStoreId = this._getStoreId(roleId, currentContextItem);
				var currentStore = this._permissionsStores[currentStoreId];
				var permissionItem = currentStore.getPermissionItem(operationId);
				if (currentStore.getPermission(permissionItem, currentContextItem) !== PermittedType.INHERITED) {
					isConfigured = true;
					currentActiveScope.permissionDefined = true;
					currentActiveScope.permissionFinal = currentStore.isFinal(permissionItem);
					winningContextItem = currentContextItem;
					winningStore = currentStore;			
					var currentContext = this._processAreaStore.getValue(currentContextItem, "enabledContext"); //$NON-NLS-1$
					if (currentContext !== ProcessConstants.PermissionContext.ITERATION_TYPE) {
						isFinalInParent = currentActiveScope.permissionFinal;
					}
				} else {
					currentActiveScope.permissionDefined = false;
					currentActiveScope.permissionFinal = false;
				}
				completeProcessingCurrentScope();
			});
			
			/*
			 * function called after determining the permission
			 * state for a scope
			 */
			var completeProcessingCurrentScope = dojo.hitch(this, function() {
				if (isFinalInParent) {
					// No need to check server permissions - customization will not be permitted
					var alwaysContext = this._editorContext === ProcessConstants.EditorContext.PROJECT
				        ? ProcessConstants.PermissionContext.PROJECT
				        : ProcessConstants.PermissionContext.TEAM;
					var winningContext; 
					var winningContextTitle;
					if (winningContextItem === this._processAreaStore.alwaysRootItem) {
						winningContext = alwaysContext;
						winningContextTitle = this._getProcessAreaTitle();
					} else {
						winningContext = this._processAreaStore.getValue(winningContextItem, "enabledContext"); //$NON-NLS-1$
						winningContextTitle = this._processAreaStore.getValue(winningContextItem, ProcessConstants.ExtensionDTO.NAME);															
					}
					this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_5);
					successCallback({
						isFinal: true,
						context: winningContext,
						contextTitle: winningContextTitle,
						areaContext: alwaysContext,
						areaContextTitle: this._getProcessAreaTitle()
					});
				} else if (nextScopeIndex == 0) {
					// The local permission state has been determined for all scopes
					if (this._isChildArea) {
						// Need to merge with permission state on server
						computeInheritedPermissionsByCallingServer();
					} else {
						computeInheritedPermissionsFromLocalData();
					}
				} else {
					nextScopeIndex--;
					checkEditorPermissionState();
				}
		    });
			
			/*
			 * This process area is not a child area and isFinal is not set in a parent scope
			 * Return permissions computed using local data.
			 */
			var computeInheritedPermissionsFromLocalData = dojo.hitch(this, function() {
				this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_5);	
				if (winningStore !== undefined) {
					successCallback({
						denialExplanation: winningStore.getDenialExplanation(
								winningStore.getPermissionItem(operationId)),
						getPermission: function(actionId) {
							var permissionItem = winningStore.getPermissionItem(actionId);
							return winningStore.getPermission(permissionItem, winningContextItem);
						}
					});
				} else {
					successCallback({
						getPermission: function(actionId) {
							return PermittedType.NOT_PERMITTED;
						}
					});
				}
			});
			
			/*
			 * This process area is a child area and isFinal is not set in a parent scope
			 * Call the server to determine the winning scope and if necessary call the 
			 * server again to load permissions data
			 */
			var computeInheritedPermissionsByCallingServer = dojo.hitch(this, function() {
				// Because the only array that can be passed to ProcessWebUIService as
				// parameters are arrays of scalars reorganize the data into that format
				var activeScopeContexts = [];
				var activeScopeContextIds = [];
				var activeScopePermissionDefined = [];
				var activeScopePermissionFinal = [];
				for (var i = 0; i < activeScopes.length; i++) {
					var nextScope = activeScopes[i];
					activeScopePermissionDefined.push(nextScope.permissionDefined);
					activeScopePermissionFinal.push(nextScope.permissionFinal);
					var context;
					var contextId;
					if (nextScope.contextItem === this._processAreaStore.alwaysRootItem) {
						// context is either TEAM or PROJECT 					
						var isProjectOperation;
						this._permissionExtensionsStore.fetchItemByIdentity({
							identity: operationId,
							onItem: dojo.hitch(this, function(item) {
								if (item !== null) {
									isProjectOperation = this.isProjectOperation(item);
								}
							})
						});
						context = isProjectOperation ? ProcessConstants.PermissionContext.PROJECT: ProcessConstants.PermissionContext.TEAM;
						contextId = this._processAreaItemId;
					} else {
					    context = this._processAreaStore.getValue(nextScope.contextItem, "enabledContext"); //$NON-NLS-1$
					    contextId = this._processAreaStore.getSavedContextId(nextScope.contextItem);
				    }
					activeScopeContexts.push(context);
					activeScopeContextIds.push(contextId);		
				}
				this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_5);	
				this._startLoadingFunction(this.messages.PermissionsWorkingCopy_6);	
				// Strip the category off the front of the operation id
				var unqualifiedOperationId = operationId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR)[1];
				var getInheritedPermissionsParameter = {
						owningApplicationKey: this._owningApplicationKey,
						processAreaContext: this._editorContext,
						processAreaItemId: this._processAreaItemId,
						roleId: roleId,
						operationId: unqualifiedOperationId,
						activeScopeContexts: activeScopeContexts,
						activeScopeContextIds: activeScopeContextIds,
						activeScopePermissionDefined: activeScopePermissionDefined,
						activeScopePermissionFinal: activeScopePermissionFinal
				};
				// Save parameters which will be needed by the response handlers
				this._getInheritedPermissionParameters = {
						successCallback: successCallback,
						errorCallback: errorCallback,
						operationId: operationId,
						roleId: roleId
					};
				var srh = new ServiceResponseHandler(this, "_getInheritedPermissionsSuccess", "_getInheritedPermissionsError"); //$NON-NLS-1$ //$NON-NLS-2$ )
				ProcessClient.getInheritedPermissions(getInheritedPermissionsParameter, srh);
			});		

			checkEditorPermissionState();
		},
		
		/*
		 * Called after successful execution of a getInheritedPermissions service call
		 */
		_getInheritedPermissionsSuccess: function(data) {
			var successCallback = this._getInheritedPermissionParameters.successCallback; 
			if (data.finalInParent) {
				this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_6);
				var winningContextTitle;
				var winningContext;
				if (ProcessConstants.PermissionContext.PROJECT === data.context || ProcessConstants.PermissionContext.TEAM === data.context) {
					winningContextTitle = data.processAreaName;
					// The customize dialog expects the context to be set to the 
					// process area kind, not the permissions kind in the "always" scope
					winningContext = data.processAreaContext;
			    } else {
			    	winningContext = data.context;
					// For anything other than PROJECT or TEAM scope look up the
					// title in the timelines store
					var query = {
							enabledContext: data.context				
					};
					if (ProcessConstants.PermissionContext.ITERATION === data.context) {
						query.originalContextId = data.contextId;
					} else {
						query.contextId = data.contextId;
					}

					this._processAreaStore.fetch({
						query: query,
						queryOptions: {
							deep: true
						},
						onItem: dojo.hitch(this, function(scopeItem) {
							winningContextTitle = this._processAreaStore.getValue(scopeItem, ProcessConstants.ExtensionDTO.NAME);
						})
					});
                }
				this._getInheritedPermissionParameters = undefined;
				successCallback({
					isFinal: true,
					context: winningContext,
					contextTitle: winningContextTitle,
					areaContext: data.processAreaContext,
					areaContextTitle: data.processAreaName
				});
			} else {
				// get a store for the winning scope
				var winningStore;
				var operationId = this._getInheritedPermissionParameters.operationId;
				if (data.processAreaId === this._processAreaItemId && data.processAreaContext === this._editorContext) {				
					// Permissions are inherited from this process area, the store will 
					// already be loaded because it was used to create the parameters 
					// to the getInheritedPermissions service call
					var roleId = this._getInheritedPermissionParameters.roleId;
					var winningContextItem;
					if (data.context == ProcessConstants.PermissionContext.PROJECT ||
						data.context == ProcessConstants.PermissionContext.TEAM) {
						winningContextItem = this._processAreaStore.alwaysRootItem;
					} else {
						// Look up the context
						var query = {
							enabledContext: data.context
						};			
						if (ProcessConstants.PermissionContext.ITERATION === data.context) {
							query.originalContextId =data.contextId;
						} else {
							query.contextId = data.contextId;
						}
						this._processAreaStore.fetch({
							query: query,
							queryOptions: {
								deep: true
							},
							onItem: dojo.hitch(this, function(item) {
								winningContextItem = item;
							})
						});
					
					}
					// The store will already be loaded because it was used to create 
					// the parameters to the getInheritedPermissions service call
					var winningStoreId = this._getStoreId(roleId, winningContextItem);
					var winningStore = this._permissionsStores[winningStoreId];

				} else {
					// Permissions will be inherited from a remote scope
					// The permission data is in the response from the server call
					winningStore = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore({
						permissionExtensionsStore: this._permissionExtensionsStore,
						isChildArea: this._isChildArea,
						processAreaStore: this._processAreaStore
					});
					this._populatePermissionStore(winningStore, data.permissions, 0);
				}
				var callbackParams = {
					isFinal: false,
					denialExplanation: winningStore.getDenialExplanation(winningStore.getPermissionItem(operationId)),
					getPermission: function(actionId) {
						var permissionItem = winningStore.getPermissionItem(actionId);
						return winningStore.getPermission(permissionItem, winningContextItem);
					}
				};
				this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_6);
				this._getInheritedPermissionParameters = undefined;
				successCallback(callbackParams);
			}
		},
		
		/*
		 * Called after unsuccessful execution of a getInheritedPermissions service call
		 */
		_getInheritedPermissionsError: function(args) {
			this._stopLoadingFunction(this.messages.PermissionsWorkingCopy_6);	
			var errorCallback = this._getInheritedPermissionParameters.errorCallback; 
			this._getInheritedPermissionParameters = undefined;
			errorCallback(args);
		},

		
		/**
		 * Clear stored permissions and references to resources such as
		 * dojo connect handles. Should be called when the working copy is
		 * no longer needed, (eg. on editor destruction), or when the
		 * load functions are to re-fetch data from the server, (eg. on editor
		 * refresh).
		 */
		clearState: function() {
			dojo.forEach(this._storeConnects, function(handle) {
				dojo.disconnect(handle);
			});
			this._storeConnects = [];
			
			//Update maps _configuredRolesByContextId and
			//_configuredScopesByRoleId that maintain the
			//state of customized roles and scopes when
			//roles or scopes are deleted
			
			this._storeConnects.push(dojo.connect(this._processAreaStore, "aboutToDelete", this, function(item) { //$NON-NLS-1$
				var contextId = this._processAreaStore.getSavedContextId(item);
				if (this._configuredRolesByContextId) {
					delete this._configuredRolesByContextId[contextId];
				}
				if (this._configuredScopesByRoleId) {
					for (var roleId in this._configuredScopesByRoleId) {
						delete this._configuredScopesByRoleId[roleId][contextId];
					}
				}
			}));
			
			this._storeConnects.push(dojo.connect(this._rolesStore, "aboutToDelete", this, function(item) { //$NON-NLS-1$
				var roleId = this._rolesStore.getValue(item, ProcessConstants.ProcessRoleDTO.ID);
				if (this._configuredScopesByRoleId) {
					delete this._configuredScopesByRoleId[roleId];
				}
				if (this._configuredRolesByContextId) {
					for (var contextId in this._configuredRolesByContextId) {
						delete this._configuredRolesByContextId[contextId][roleId];
					}
				}
			}));
			
			this._loadingPermissionExtensions = 0;
			this._permissionExtensionsData = {
				identifier: ProcessConstants.ExtensionDTO.ID,
				label: ProcessConstants.ExtensionDTO.NAME,
				items: []
			};
			delete this._permissionExtensionsStore;
			this._permissionExtensionCategoriesByName = {};
			this._permissionExtensionsIdMap = {};
			
			this._loadingPermissions = 0;
			this._loadingPermissionsForAllRoles = 0;
			
			this._permissionsStores = {};
			this._permissionsStoreInfos = {};			
			this._loadedPermissions = {};
			this._pausedNotificationStores = [];
			
			delete this._configuredScopes;
			
			delete this._pendingPermissions;
			delete this._pendingPermissionsForAllRoles;
		}
	});
	
	com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy.PROP_PARENT_ID = PROP_PARENT_ID;
})();
